<?php
/**
*
* @package phpBB3 - Facebook Connect MOD
* @version $Id: functions_fbconnect.php 1 2009-07-29 20:47:00Z DeViAnThans3 $
* @copyright (c) 2009 DeViAnThans3
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

//ini_set('display_errors',1);
//error_reporting(E_ALL);

require_once($phpbb_root_path . "includes/acm/acm_file.".$phpEx);

/**
* Get the correct language ISO code (for automatic language detection)
*/
function get_valid_lang($locale)
{
	global $db, $config;
	
	// Not enabled? Return default..
	if($config['fbconnect_autolang'] == 0)
	{
		return $config['default_lang'];
	}
	
	// Get all available languages
	$langs = array();
	$result = $db->sql_query("SELECT lang_iso FROM " . LANG_TABLE);
	while($row = $db->sql_fetchrow($result))
	{
		$langs[] = $row['lang_iso'];
	}
	// LL_CC installed!
	$lang = strtolower($locale);
	if(in_array($lang, $langs))
	{
		return $lang;
	}
	
	// LL installed!
	$lang = strtolower(substr($locale, 0, strpos($locale, '_')));
	if(in_array($lang, $langs))
	{
		return $lang;
	}
	
	// None installed - return default value
	return $config['default_lang'];
} 

/**
* Add an existing user to the facebook group if this option is enabled (and he isn't member yet)
*/
function add_exist_group($userid)
{
	global $db, $config;
	
	if($config['fbconnect_existgroup'])
	{
		$sql = 'SELECT COUNT(*) AS total FROM ' . USER_GROUP_TABLE . ' WHERE user_id = \'' . $userid . '\' AND group_id = \'' . $config['fbconnect_usergroup'] . '\'';
		$result = $db->sql_query($sql);
		$t = (int) $db->sql_fetchfield('total');
		
		if($t==0)
		{
			// Place into appropriate group...
			$sql = 'INSERT INTO ' . USER_GROUP_TABLE . ' ' . $db->sql_build_array('INSERT', array(
				'user_id'		=> (int) $userid,
				'group_id'		=> (int) $config['fbconnect_usergroup'],
				'user_pending'	=> 0)
			);
			return $db->sql_query($sql);
		}
	}
	
	return false;
}

/**
* Automatically befriend users who are also befriended on Facebook (if configured)
*/
function connect_friends($userid, $friends)
{
	global $db, $config;
	
	if($config['fbconnect_friendimport'] == "1")
	{
		// Connect friends
		$sql = "SELECT fb_id, user_id FROM " . FBIDS_TABLE;
		$result = $db->sql_query($sql);
		while($c = $db->sql_fetchrow($result))
		{                                                                  
			if(in_array($c['fb_id'], $friends))
			{                   
				$sql = 'SELECT COUNT(*) AS total FROM ' . ZEBRA_TABLE . ' WHERE user_id = \'' . $userid . '\' AND zebra_id = \'' . $c['user_id'] . '\'';
				$nresult = $db->sql_query($sql);
				$t = (int) $db->sql_fetchfield('total');
				if($t == 0)
				{
					$sql = 'INSERT INTO ' . ZEBRA_TABLE . ' ' . $db->sql_build_array('INSERT', array(
						'user_id'    => (int) $userid,
						'zebra_id'    => (int) $c['user_id'],
						'friend'	=> 1)
					);  
                    $db->sql_query($sql);  
				}
				
				
				$sql = 'SELECT COUNT(*) AS total FROM ' . ZEBRA_TABLE . ' WHERE zebra_id = \'' . $userid . '\' AND user_id = \'' . $c['user_id'] . '\'';
				$nresult = $db->sql_query($sql);
				$t = (int) $db->sql_fetchfield('total');
				if($t == 0)
				{
					$sql = 'INSERT INTO ' . ZEBRA_TABLE . ' ' . $db->sql_build_array('INSERT', array(
						'zebra_id'    => (int) $userid,
						'user_id'    => (int) $c['user_id'],
						'friend'	=> 1)
					); 
                    $db->sql_query($sql);
				}
				
			}           
		}
	}
}

/**
* Unlink an existing user who doesn't get deleted!
*/
function fb_unlink($user_id)
{
    global $db;
    
    // Now, let's remove the link!
    @$db->sql_query("DELETE FROM " . FBIDS_TABLE . " WHERE user_id = " . $user_id);
}

/**
* Link a Facebook ID to a User ID
*/
function link_user($user_id, $fb_id, $edit = 0)
{
	global $db;
	
	// Let's first check if the account isn't linked already. (security measure)
	$sql = 'SELECT COUNT(*) AS total FROM ' . FBIDS_TABLE . ' WHERE user_id = \'' . $user_id . '\' OR fb_id = \'' . $fb_id . '\'';
	$result = $db->sql_query($sql);
	$c = (int) $db->sql_fetchfield('total');
	if($c > 0)
	{
		die("<b>Fatal Error:</b> Attempt to link an already linked account.");
	}
	
	// Link user to facebook account!
	$sql = 'INSERT INTO ' . FBIDS_TABLE . ' ' . $db->sql_build_array('INSERT', array(
		'user_id'    => (int) $user_id,
		'fb_id'    => $fb_id,
		'edit'	=>	$edit)
	);
	$db->sql_query($sql);
}

/**
* Get the Facebook Language code for the current language code
*/
function get_fb_lang($langkey = false)
{
	global $config;
	
	if($langkey == false)
	{
		$langkey = $config['default_lang'];
	}
	
	$langpairs = array(
		'ar' 	=> 	'ar_AR',
		'id' 	=> 	'id_ID',
		'ca' 	=> 	'ca_ES',
		'cs' 	=> 	'cs_CZ',
		'da'	=>	'da_DK',
		'de'	=> 	'de_DE',
		'de_x_sie'	=>	'de_DE',
		'et'	=>	'et_EE',
		'en_us'	=>	'en_US',
		'es'	=>	'es_LA',
		'eu'	=>	'eu_ES',
		'fr'	=>	'fr_FR',
		'gl'	=>	'gl_ES',
		'hr'	=>	'hr_HR',
		'it'	=>	'it_IT',
		'lt'	=>	'lt_LT',
		'hu'	=>	'hu_HU',
		'nl'	=>	'nl_NL',
		'nl_x_formal'	=>	'nl_NL',
		'nb'	=>	'nb_NO',
		'pl'	=>	'pl_PL',
		'pt'	=>	'pt_PT',
		'pt_br'	=>	'pt_BR',
		'ro'	=>	'ro_RO',
		'sr'	=>	'sr_RS',
		'sr_x_cyr'	=>	'sr_RS',
		'sq'	=>	'sq_AL',
		'sk'	=>	'sk_SK',
		'sl'	=>	'sl_SL',
		'fi'	=>	'fi_FI',
		'sv'	=>	'sv_SE',
		'vi'	=>	'vi_VN',
		'tr'	=>	'tr_TR',
		'el'	=>	'el_GR',
		'be'	=>	'ru_RU',
		'ru'	=>	'ru_RU',
		'uk'	=>	'uk_UA',
		'he'	=>	'he_IL',
		'fa'	=>	'fa_IR',
		'th'	=>	'th_TH',
		'ja'	=>	'ja_JP',
		'zh_cmn_hant'	=>	'zh_CN',
		'zh_cmn_hans'	=>	'zh_CN'
	);
	
	// Unmatched language: ku (Kurdian)
	// Unmatched languages will default to en_GB.
	if(isset($langpairs[strtolower($langkey)]))
	{
		return $langpairs[strtolower($langkey)];
	}else{
		return '';
	}
}

/**
* Fetch Facebook data and inject it into database results.
*/
function inject_fb_data(&$ud, $uid = 0)
{
    global $config, $db, $user, $phpEx, $phpbb_root_path;
    
    // Do not import if disabled.
    if($config['fbconnect_dataimport'] == "0")
    {
        return;
    }
    
    if($uid != 0)
    {
        $ud['user_id'] = $uid;
    }
    
    // Set up Facebook API client
    require_once($phpbb_root_path . 'includes/fbdata.'.$phpEx);
    $fbdatao = new fbdata();
    
    $sql = "SELECT user_id, fb_id FROM " . FBIDS_TABLE . " WHERE user_id = '{$ud['user_id']}'";
    $result = $db->sql_query($sql);
    $fbi = 0;
    while($row = $db->sql_fetchrow($result))
    {
        $fbi = $row['fb_id'];
    }
    
    $fbdata = $fbdatao->obtain_fbdata($fbi);
    
    if(!is_array($fbdata))
    {
        return;
    }
    
    // Data fetch success. Let's import the info!
    $uid = $ud['user_id'];
    
    // Some general data
    $ud['user_sig'] = (trim($ud['user_sig']) == "") ? $fbdata['profile_blurb'] : $ud['user_sig'];
    $ud['user_website'] = (trim($ud['user_website']) == "") ? $fbdata['profile_url'] : $ud['user_website'];
    $ud['user_interests'] = (trim($ud['user_interests']) == "") ? $fbdata['interests'] : $ud['user_interests'];
    
    // Location
    if($fbdata['current_location']['city'] != "" && $fbdata['current_location']['country'] != "")
    {
        $ud['user_from'] = (trim($ud['user_from']) == "") ? $fbdata['current_location']['city'] . ", " . $fbdata['current_location']['country'] : $ud['user_from'];
    }elseif($fbdata['current_location']['country'] != ""){
        $ud['from'] = (trim($ud['user_from']) == "") ? $fbdata['current_location']['country'] : $ud['user_from'];
    }elseif($fbdata['current_location']['city'] != ""){
        $ud['from'] = (trim($ud['user_from']) == "") ? $fbdata['current_location']['city'] : $ud['user_from'];
    }
    
    // Birthday
    preg_match_all('/[0-9]{4}/', $fbdata['birthday_date'], $years, PREG_PATTERN_ORDER);
    if (count($years[0])>0 && $config['allow_birthdays'] && (trim($ud['user_birthday']) == "" || trim($ud['user_birthday']) == "0- 0-   0"))
    {
        $ud['user_birthday'] = date("j-n-Y", strtotime($fbdata['birthday_date']));
        
    }
    
    // Avatar
    if($config['allow_avatar_remote'] == "1" && trim($ud['user_avatar']) == "")
    {
        
        $avatardata = array();
        $size = getjpegsize($fbdata[$config['fbconnect_avatartype']]);
        if(is_array($size))
        {
            if($size[0] != null && $size[1] != null)
            {
                $avatardata['width'] = $size[0];
                $avatardata['height'] = $size[1];
            }else{
                $avatardata['width'] = $avatardata['height'] = 0;
            }
            $sizecheck = true; 
            if($config['fbconnect_forceavatar']=="0")
            {
                if(intval($avatardata['width']) > intval($config['avatar_max_width']) || intval($avatardata['width']) < intval($config['avatar_min_width']))
                {
                    $sizecheck = false;
                }else{
                    if(intval($avatardata['height']) > intval($config['avatar_max_height']) || intval($avatardata['height']) < intval($config['avatar_min_height']))
                    {
                        $sizecheck = false;
                    }
                }
            }
            
            if($sizecheck)
            {
                $avatardata['remotelink'] = $fbdata[$config['fbconnect_avatartype']];
                list($ud['user_avatar_type'], $ud['user_avatar'], $ud['user_avatar_width'], $ud['user_avatar_height']) = array(AVATAR_REMOTE, $avatardata['remotelink'], $avatardata['width'], $avatardata['height']);
            }
        }
    }
        
    return;
}

/**
* Get JPG file dimensions without downloading the complete remote image.
* Credit goes to james.relyea@zifiniti.com (code from php.net)
*/
function getjpegsize($img_loc) {
    $handle = fopen($img_loc, "rb");
    if(!$handle)
    {
        return false;
    }
    $new_block = NULL;
    if(!feof($handle)) {
        $new_block = fread($handle, 32);
        $i = 0;
        if($new_block[$i]=="\xFF" && $new_block[$i+1]=="\xD8" && $new_block[$i+2]=="\xFF" && $new_block[$i+3]=="\xE0") {
            $i += 4;
            if($new_block[$i+2]=="\x4A" && $new_block[$i+3]=="\x46" && $new_block[$i+4]=="\x49" && $new_block[$i+5]=="\x46" && $new_block[$i+6]=="\x00") {
                // Read block size and skip ahead to begin cycling through blocks in search of SOF marker
                $block_size = unpack("H*", $new_block[$i] . $new_block[$i+1]);
                $block_size = hexdec($block_size[1]);
                while(!feof($handle)) {
                    $i += $block_size;
                    $new_block .= fread($handle, $block_size);
                    if($new_block[$i]=="\xFF") {
                        // New block detected, check for SOF marker
                        $sof_marker = array("\xC0", "\xC1", "\xC2", "\xC3", "\xC5", "\xC6", "\xC7", "\xC8", "\xC9", "\xCA", "\xCB", "\xCD", "\xCE", "\xCF");
                        if(in_array($new_block[$i+1], $sof_marker)) {
                            // SOF marker detected. Width and height information is contained in bytes 4-7 after this byte.
                            $size_data = $new_block[$i+2] . $new_block[$i+3] . $new_block[$i+4] . $new_block[$i+5] . $new_block[$i+6] . $new_block[$i+7] . $new_block[$i+8];
                            $unpacked = unpack("H*", $size_data);
                            $unpacked = $unpacked[1];
                            $height = hexdec($unpacked[6] . $unpacked[7] . $unpacked[8] . $unpacked[9]);
                            $width = hexdec($unpacked[10] . $unpacked[11] . $unpacked[12] . $unpacked[13]);
                            return array($width, $height);
                        } else {
                            // Skip block marker and read block size
                            $i += 2;
                            $block_size = unpack("H*", $new_block[$i] . $new_block[$i+1]);
                            $block_size = hexdec($block_size[1]);
                        }
                    } else {
                        return FALSE;
                    }
                }
            }
        }
    }
    return FALSE;
}


?>